#ifndef __AIO_H
#define __AIO_H

#undef FUNC
#ifndef SWIGGY
#define FUNC(x) (*x) 
#else
#define FUNC(x) x
#endif

typedef enum {
  AIO_ADC=0, ADIO_DAC=1
} AIOType;

typedef struct AIO AIO;
typedef struct AIOAPI AIOAPI;
typedef struct VoltageRange VoltageRange;
typedef struct VoltageSubRange VoltageSubRange;
typedef struct PeriodRange PeriodRange;

struct VoltageSubRange {
  float start, step;
  unsigned count;
};
struct VoltageRange {
  VoltageSubRange low,high;
};
struct ActualVoltageRange {
  float low, high;
};

struct PeriodRange {
  unsigned start, step, count;
};
#define AIO_PR(start,step,count) ((PeriodRange){start,step,count});
#define AIO_VR(lowstart,lowstep,lowcount,highstart,highstep,highcount) ((VoltageRange){(VoltageSubRange){lowstart,lowstep,lowcount},(VoltageSubRange){,highstart,highstep,highcount}})
struct AIOAPI {
  void *FUNC(Init)(void *me);
  void FUNC(Fini)(void *me);
  int FUNC(Lock)(void *me,unsigned num,int flags);
  int FUNC(Unlock)(void *me,unsigned num,int flags);
  int FUNC(Preempt)(void *me);
  AIOType FUNC(Type)(void *me);
  int FUNC(VoltageRangeListLength)(void *me);
  int FUNC(VoltageRange)(void *me,int i,VoltageRange *rr);
  int FUNC(PrecisionList)(void *me);
  int FUNC(PeriodRangeListLength)(void *me);
  int FUNC(PeriodRange)(void *me,int i,PeriodRange *r);
  int FUNC(TriggerListLength)(void *me);
  int FUNC(TriggerList)(void *me,int *tlist);
  int FUNC(Channels)(void *me);
  int FUNC(Configuration)(void *me,float *low,float *high,int *prec,int *rate,int *trigger,int *itrig);
  int FUNC(ConfigureTest)(void *me,int count,float *low,float *high,int *prec,int *rate,int *trigger,int itrig);
  int FUNC(Configure)(void *me,int count,float *low,float *high,int *prec,int *rate,int *trigger,int itrig);
  int FUNC(ITrig)(void *me,int itrig);
  int FUNC(Get)(void *me,int channel);
  int FUNC(Put)(void *me,int channel,int value);
  int FUNC(Ready)(void *me,int channel);
  int FUNC(Gets)(void *me,int channelMask,int samples,void *mebuf);
  int FUNC(Puts)(void *me,int channelMask,int samples,void *mebuf);
  int FUNC(Readys)(void *me,int channelMask,int samples,void *mebuf);
};

struct AIO {
  void *FUNC(Init)(void *me);
  void FUNC(Fini)(void *me);
  int FUNC(Lock)(void *me,unsigned num,int flags);
  int FUNC(Unlock)(void *me,unsigned num,int flags);
  int FUNC(Preempt)(void *me);
  AIOType FUNC(Type)(void *me);
  int FUNC(VoltageRangeListLength)(void *me);
  int FUNC(VoltageRange)(void *me,int i,VoltageRange *rr);
  int FUNC(PrecisionList)(void *me);
  int FUNC(PeriodRangeListLength)(void *me);
  int FUNC(PeriodRange)(void *me,int i,PeriodRange *r);
  int FUNC(TriggerListLength)(void *me);
  int FUNC(TriggerList)(void *me,int *tlist);
  int FUNC(Channels)(void *me);
  int FUNC(Configuration)(void *me,float *low,float *high,int *prec,int *rate,int *trigger,int *itrig);
  int FUNC(ConfigureTest)(void *me,int count,float *low,float *high,int *prec,int *rate,int *trigger,int itrig);
  int FUNC(Configure)(void *me,int count,float *low,float *high,int *prec,int *rate,int *trigger,int itrig);
  int FUNC(ITrig)(void *me,int itrig);
  int FUNC(Get)(void *me,int channel);
  int FUNC(Put)(void *me,int channel,int value);
  int FUNC(Ready)(void *me,int channel);
  int FUNC(Gets)(void *me,int channelMask,int samples,void *mebuf);
  int FUNC(Puts)(void *me,int channelMask,int samples,void *mebuf);
  int FUNC(Readys)(void *me,int channelMask,int samples,void *mebuf);
  int InitStatus;
};

#define AIO_HZ(hz) ((int)(1000000000/hz))
enum {
  NetAIO_Lock=0,
  NetAIO_VoltageRangeListLength=1,
  NetAIO_VoltageRange=2,
  NetAIO_PrecisionList=3,
  NetAIO_RateRangeListLength=4,
  NetAIO_RateRange=5,
  NetAIO_TriggerListLength=6,
  NetAIO_TriggerList=7,
  NetAIO_Channels=8,
  NetAIO_Configuration=9,
  NetAIO_ConfigureTest=10,
  NetAIO_Configure=11,
  NetAIO_Get=12,
  NetAIO_Put=13,
  NetAIO_Ready=14,
  NetAIO_Gets=15,
  NetAIO_Puts=16,
  NetAIO_Readys=17
};
#define AIO_APICount (1+NetAIO_Readys)

#endif
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.

