#ifndef __WindowBus_h
#define __WindowBus_h
#include "Bus.h"

typedef struct WindowBus WindowBus;

struct WindowBus {
  void *FUNC(Init)(void *me,...);
  void FUNC(Fini)(void *me);
  int FUNC(Lock)(void *me,unsigned num,int flags);
  int FUNC(Unlock)(void *me,unsigned num,int flags);
  int FUNC(Preempt)(void *me);
  unsigned char FUNC(Peek8)(void *me,int adrs);
  void FUNC(Poke8)(void *me,int adrs,unsigned char val);
  unsigned short FUNC(Peek16)(void *me,int adrs);
  void FUNC(Poke16)(void *me,int adrs,unsigned short val);
  unsigned FUNC(Peek32)(void *me,int adrs);
  void FUNC(Poke32)(void *me,int adrs,unsigned val);
  int FUNC(BitGet8)(void *me,int adrs,int bit);
  void FUNC(BitAssign8)(void *me,int adrs,int bit,int val);
  void FUNC(BitSet8)(void *me,int adrs,int bit);
  void FUNC(BitClear8)(void *me,int adrs,int bit);
  int FUNC(BitGet16)(void *me,int adrs,int bit);
  void FUNC(BitAssign16)(void *me,int adrs,int bit,int val);
  void FUNC(BitSet16)(void *me,int adrs,int bit);
  void FUNC(BitClear16)(void *me,int adrs,int bit);
  int FUNC(BitGet32)(void *me,int adrs,int bit);
  void FUNC(BitAssign32)(void *me,int adrs,int bit,int val);
  void FUNC(BitSet32)(void *me,int adrs,int bit);
  void FUNC(BitClear32)(void *me,int adrs,int bit);
  void FUNC(PeekStream)(void *me,int adr,int dir,char *buf,int len);
  void FUNC(PokeStream)(void *me,int adr,int dir,char *buf,int len);
  void FUNC(Refresh)(void *me);
  void FUNC(Commit)(void *me,int forceall);
  int FUNC(BitToggle8)(void *me,int adrs,int bit);
  int FUNC(BitToggle16)(void *me,int adrs,int bit);
  int FUNC(BitToggle32)(void *me,int adrs,int bit);
  unsigned char FUNC(Assign8X)(void *me,int adrs,int bit1,int bit0,int val);
  unsigned short FUNC(Assign16X)(void *me,int adrs,int bit1,int bit0,int val);
  unsigned FUNC(Assign32X)(void *me,int adrs,int bit1,int bit0,int val);
  unsigned char FUNC(BitsGet8)(void *me,int adrs,int bit1,int bit0);
  unsigned short FUNC(BitsGet16)(void *me,int adrs,int bit1,int bit0);
  unsigned FUNC(BitsGet32)(void *me,int adrs,int bit1,int bit0);
  int InitStatus;
  unsigned LockBase;
  int deferlock;
  Bus *sub; // bus containing the window registers
  int adrs, data; // address of adrs and data registers, respectively
};

void *WindowBusInit(WindowBus *,void *bus,int adrs,int data);
void *WindowBusInit2(WindowBus *,void *mem,int Pages);
void WindowBusFini(WindowBus *);
unsigned char WindowBusPeek8(WindowBus *,int adrs);
void WindowBusPoke8(WindowBus *,int adrs,unsigned char val);
unsigned short WindowBusPeek16(WindowBus *,int adrs);
void WindowBusPoke16(WindowBus *,int adrs,unsigned short val);
unsigned WindowBusPeek32(WindowBus *,int adrs);
void WindowBusPoke32(WindowBus *,int adrs,unsigned val);
int WindowBusBitGet8(WindowBus *bus,int adrs,int bit);
void WindowBusBitAssign8(WindowBus *bus,int adrs,int bit,int val);
void WindowBusBitSet8(WindowBus *bus,int adrs,int bit);
void WindowBusBitClear8(WindowBus *bus,int adrs,int bit);
int WindowBusBitGet16(WindowBus *bus,int adrs,int bit);
void WindowBusBitAssign16(WindowBus *bus,int adrs,int bit,int val);
void WindowBusBitSet16(WindowBus *bus,int adrs,int bit);
void WindowBusBitClear16(WindowBus *bus,int adrs,int bit);
int WindowBusBitGet32(WindowBus *bus,int adrs,int bit);
void WindowBusBitAssign32(WindowBus *bus,int adrs,int bit,int val);
void WindowBusBitSet32(WindowBus *bus,int adrs,int bit);
void WindowBusBitClear32(WindowBus *bus,int adrs,int bit);
void WindowBusPeekStream(WindowBus *bus,int adr,int dir,char *buf,int len);
void WindowBusPokeStream(WindowBus *bus,int adr,int dir,char *buf,int len);
void WindowBusRefresh(WindowBus *);
void WindowBusCommit(WindowBus *,int forceall);
int WindowBusBitToggle8(WindowBus *bus,int adrs,int bit);
int WindowBusBitToggle16(WindowBus *bus,int adrs,int bit);
int WindowBusBitToggle32(WindowBus *bus,int adrs,int bit);
unsigned char WindowBusAssign8X(WindowBus *,int adrs,int bit1,int bit0,int val);
unsigned short WindowBusAssign16X(WindowBus *,int adrs,int bit1,int bit0,int val);
unsigned WindowBusAssign32X(WindowBus *,int adrs,int bit1,int bit0,int val);
unsigned char WindowBusBitsGet8(WindowBus *,int adrs,int bit1,int bit0);
unsigned short WindowBusBitsGet16(WindowBus *,int adrs,int bit1,int bit0);
unsigned WindowBusBitsGet32(WindowBus *,int adrs,int bit1,int bit0);

#endif
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
