#include <stdio.h>
#include <string.h>
#include "libtsctl.h"
#include "ts/Net.h"

void hprintf(int h,char *format, ...) {
  va_list ap;
  char dummy[1];
  int len;

  va_start(ap,format);
  len = vsnprintf(dummy,0,format,ap);
  {
    char str[len];
    vsprintf(str,format,ap);
    write(h,str,len);
  }
}

typedef struct tsctlCommand tsctlCommand;
enum TypeEnum {
  T_Void, T_Integer32, T_VectorInteger32, T_BuildInfo, T_VectorInteger8,
  T_Pair
};
enum {
  MustNoLock = 0,
  MustReadLock,
  MustWriteLock
};

struct tsctlCommand {
  int Class, Inst;
  int Command,OpCode,Extra,MustLock, Lock;
  int Status;
  System *sys;
  union {
    Bus *bus;
    Time *time;
    Pin *pin;
    DIORaw *dioraw;
    DIO *dio;
    TWI *twi;
    CAN *can;
    SPI *spi;
    // EDIO *edio;
    AIO *aio;
  };
  enum TypeEnum rType;
  union {
    int rInteger32;
    char *rVectorInteger8;
    int *rVectorInteger32;
    struct BuildInfo *rBuildInfo;
    struct {
      char *key;
      int value;
    } rPair;
  };
};
enum {
  CommandErrorClass=-100, CommandErrorArgCount, CommandErrorCommandBad,
  CommandErrorArgsBad, CommandErrorNoSuchObject, CommandErrorObjectInit,
  CommandErrorLock=-200
};


char *Class[] = {
  "System", "Bus", "Time", "Pin", "DIORaw", "DIO", "TWI", "CAN", "SPI", 0
};

char *SystemCommandNames[] = {
  "ClassCount", "InstanceCount", "APICount", 
  "LockCount", "LockHolderInfo", "ConnWaitInfo", "CANBusGet", "BuildInfo",
  "ModelId", "BaseBoardId", "MapLength", "MapGet", "MapLookup", 
  "MapLookupPartial", "MapAdd", "MapDelete",
  0
};

char *BusCommandNames[] = {
  "Peek8", "Poke8", "Peek16", "Poke16", "Peek32",
  "Poke32", "BitGet8", "BitAssign8", "BitSet8", "BitClear8", "BitGet16",
  "BitAssign16", "BitSet16", "BitClear16", "BitGet32", "BitAssign32",
  "BitSet32", "BitClear32", "PeekStream", "PokeStream", "Refresh", "Commit",
  "BitToggle8", "BitToggle16", "BitToggle32", "Assign8X", "Assign16X",
  "NetBus_Assign32X", "BitsGet8", "BitsGet16", "BitsGet32", 0
};
char *TimeCommandNames[] = {
  "Wait", "Delay", "Tick", "usElapsed", "usFuture", "TimeoutQ", "TPS", 0
};
char *PinCommandNames[] = {
  "ModeGet", "ModeSet", 0
};
char *DIORawCommandNames[] = {
  "DirSet", "DataSet", "DirGet", "DataGet", "Count", 0
};
char *DIOCommandNames[] = {
  "Refresh", "Commit", "Set", "Get", 
  "SetAsync", "GetAsync", "Wait", "Count", "Capabilities", 0
};

char *TWICommandNames[] = {
  "Write","Read", 0
};

char *CANCommandNames[] = {
  "Rx", "Tx", "BaudSet", "BaudGet", 0
};

char *SPICommandNames[] = {
  "Write","Read","ReadWrite","ClockSet","EdgeSet", 0
};

int SystemCommandOpcode[] = {
  NetSystem_ClassCount, NetSystem_InstanceCount, NetSystem_APICount,
  NetSystem_LockCount, NetSystem_LockHolderInfo, NetSystem_ConnWaitInfo,
  NetSystem_CANBusGet, NetSystem_BuildInfo, NetSystem_ModelId, 
  NetSystem_BaseBoardId, NetSystem_MapLength, NetSystem_MapGet,
  NetSystem_MapLookup, NetSystem_MapLookupPartial, NetSystem_MapAdd,
  NetSystem_MapDelete
};

int BusCommandOpcode[] = {
  NetBus_Peek8, NetBus_Poke8, 
  NetBus_Peek16, NetBus_Poke16, NetBus_Peek32, NetBus_Poke32, 
  NetBus_BitGet8, NetBus_BitAssign8, NetBus_BitSet8, NetBus_BitClear8, 
  NetBus_BitGet16, NetBus_BitAssign16, NetBus_BitSet16, NetBus_BitClear16, 
  NetBus_BitGet32, NetBus_BitAssign32, NetBus_BitSet32, NetBus_BitClear32, 
  NetBus_PeekStream, NetBus_PokeStream, NetBus_Refresh, NetBus_Commit, 
  NetBus_BitToggle8, NetBus_BitToggle16, NetBus_BitToggle32,
  NetBus_Assign8X, NetBus_Assign16X, NetBus_Assign32X,
  NetBus_BitsGet8, NetBus_BitsGet16, NetBus_BitsGet32
};
int BusMustLock[] = {
  MustReadLock, MustWriteLock,
  MustReadLock, MustWriteLock,
  MustReadLock, MustWriteLock,
  MustReadLock, MustWriteLock, MustWriteLock, MustWriteLock,
  MustReadLock, MustWriteLock, MustWriteLock, MustWriteLock,
  MustReadLock, MustWriteLock, MustWriteLock, MustWriteLock,
  MustReadLock, MustWriteLock, MustNoLock, MustNoLock,
  MustWriteLock, MustWriteLock, MustWriteLock,
  MustWriteLock, MustWriteLock, MustWriteLock,
  MustReadLock, MustReadLock, MustReadLock
};

int TimeCommandOpcode[] = {
  NetTime_Wait, NetTime_Delay, NetTime_Tick, NetTime_usElapsed,
  NetTime_usFuture, NetTime_TimeoutQ, NetTime_TPS
};
int PinCommandOpcode[] = {
  NetPin_ModeGet, NetPin_ModeSet
};
int PinMustLock[] = {
  MustReadLock, MustWriteLock
};

int DIORawCommandOpcode[] = {
  NetDIORaw_DirSet, NetDIORaw_DataSet, NetDIORaw_DirGet, 
  NetDIORaw_DataGet, NetDIORaw_Count
};
int DIORawMustLock[] = {
  MustWriteLock, MustWriteLock,
  MustReadLock, MustReadLock, MustNoLock
};

int DIOCommandOpcode[] = {
  NetDIO_Refresh, NetDIO_Commit, NetDIO_Set, NetDIO_Get, 
  NetDIO_SetAsync, NetDIO_GetAsync, NetDIO_Wait, NetDIO_Count,
  NetDIO_Capabilities
};
int DIOMustLock[] = {
  MustNoLock, MustNoLock, MustWriteLock, MustReadLock,
  MustWriteLock, MustReadLock, MustReadLock, MustNoLock, MustNoLock
};

int TWICommandOpcode[] = {
  NetTWI_Write, NetTWI_Read
};
int TWIMustLock[] = {
  MustWriteLock, MustWriteLock
};

int CANCommandOpcode[] = {
  NetCAN_Rx, NetCAN_Tx, NetCAN_BaudSet, NetCAN_BaudGet
};

int SPICommandOpcode[] = {
  NetSPI_Write, NetSPI_Read, NetSPI_ReadWrite,
  NetSPI_ClockSet, NetSPI_EdgeSet
};
int SPIMustLock[] = {
  MustWriteLock, MustWriteLock, MustWriteLock,
  MustWriteLock, MustWriteLock
};

int SystemCommandOpts[] = {
  0,1,1,0,0,1,1,0,0,0,0,1,1,2,2,1
};

int BusCommandOpts[] = {
  1,2,1,2,1,2,2,3,2,2,2,3,2,2,2,3,2,2,3,3,0,1,2,2,2,4,4,4,3,3,3
};
int TimeCommandOpts[] = {
  1,1,0,1,2,2,0
};
int PinCommandOpts[] = {
  1,2
};
int DIORawCommandOpts[] = {
  2,2,1,1,0
};
int DIOCommandOpts[] = {
  0,1,2,1,2,1,3,0,1
};
int TWICommandOpts[] = {
  4,4
};
int CANCommandOpts[] = {
  0,1,1,0
};
int SPICommandOpts[] = {
  2,2,2,1,1
};

char *SystemCommandDesc[] = {
  "", // ClassCount
  "<class#>", // InstanceCount
  "<class#>", // APICount
  "", //LockCount
  "", //LockHolderInfo
  "<Max>", //ConnWaitInfo
  "<CANInst>", // CANBusGet
  "", // BuildInfo
  "", // ModelId
  "" // BaseBoardId
  "", // MapLength
  "<index>", // MapGet
  "<name>", // MapLookup
  "<name> <value>", // MapLookupPartial
  "<name> <value>", //MapAdd
  "<name>" // MapDelete
};

char *BusCommandDesc[] = {
  "<address>", // Peek8
  "<address> <8-bit value>", // Poke8
  "<address>", // Peek16
  "<address> <16-bit value>", // Poke16
  "<address>", // Peek32
  "<address> <32-bit value>", // Poke32
  "<address> <bit>", // BitGet8
  "<address> <bit> <0|1>", // BitAssign8
  "<address> <bit>", // BitSet8
  "<address> <bit>", // BitClear8
  "<address> <bit>", // BitGet16
  "<address> <bit> <0|1>", // BitAssign16
  "<address> <bit>", // BitSet16
  "<address> <bit>", // BitClear16
  "<address> <bit>", // BitGet32
  "<address> <bit> <0|1>", // BitAssign32
  "<address> <bit>", // BitSet32
  "<address> <bit>", // BitClear32
  "<address> <direction> <length>", // PeekStream
  "<address> <direction> <byte array>", // PokeStream
  "", // Refresh
  "<forceall flag>", // Commit
  "<address> <bit>", // BitToggle8
  "<address> <bit>", // BitToggle16
  "<address> <bit>", // BitToggle32
  "<address> <bit1> <bit0> <value>", // Assign8X
  "<address> <bit1> <bit0> <value>", // Assign16X
  "<address> <bit1> <bit0> <value>", // Assign32X
  "<address> <bit1> <bit0>", // BitsGet8
  "<address> <bit1> <bit0>", // BitsGet16
  "<address> <bit1> <bit0>" // BitsGet32
};
char *TimeCommandDesc[] = {
  "<microseconds>", // Wait
  "<microseconds>", // Delay
  "", // Tick
  "<start>", // usElapsed
  "<start> <us>", // usFuture
  "<start> <end>", // TimeoutQ
  "" // TPS
};
char *PinCommandDesc[] = {
  "<pin number>", // ModeGet
  "<pin number> <mode>", // ModeSet
};
char *DIORawCommandDesc[] = {
  "<num> <asOutput>", // DirSet
  "<num> <asHigh>", // DataSet
  "<num>", // DirGet
  "<num>", // DataGet
  "" // Count
};
char *DIOCommandDesc[] = {
  "", // Refresh
  "<forceall flag>", // Commit
  "<DIO number> <state>", // Set
  "<DIO number>", // Get
  "<DIO number> <state>", // SetAsync
  "<DIO number>", // GetAsync
  "<min> <max> <DIONum:...> <DIONum:...>", // Wait
  "", // Count
  "<DIO number>" // Capabilities
};
char *TWICommandDesc[] = {
  "<devadr> <adrslen> <adrs> <byte array>", // Write
  "<devadr> <adrslen> <adrs> <length>" // Read
};
char *CANCommandDesc[] = {
  "", // Rx
  "<byte array>", // Tx
  "<baudrate>", // BaudSet
  "", // BaudGet
};
char *SPICommandDesc[] = {
  "<adrs> <byte array>", // Write
  "<adrs> <length>", // Read
  "<adrs> <byte array>", // ReadWrite
  "<clock freq>", // ClockSet
  "<edge>" // EdgeSet
};

char **CommandNames[] = {
  SystemCommandNames, BusCommandNames, TimeCommandNames, PinCommandNames, 
  DIORawCommandNames, DIOCommandNames, TWICommandNames, CANCommandNames, 
  SPICommandNames, 0
};
int *CommandOpts[] = {
  SystemCommandOpts, BusCommandOpts, TimeCommandOpts, PinCommandOpts, 
  DIORawCommandOpts, DIOCommandOpts, TWICommandOpts, CANCommandOpts,
  SPICommandOpts
};
char **CommandDesc[] = {
  SystemCommandDesc, BusCommandDesc, TimeCommandDesc, PinCommandDesc,
  DIORawCommandDesc, DIOCommandDesc, TWICommandDesc, CANCommandDesc,
  SPICommandDesc
};
int *CommandOpcode[] = {
  SystemCommandOpcode, BusCommandOpcode, TimeCommandOpcode, PinCommandOpcode,
  DIORawCommandOpcode, DIOCommandOpcode, TWICommandOpcode, CANCommandOpcode,
  SPICommandOpcode
};

int *CommandMustLock[] = {
  0, BusMustLock, 0, PinMustLock,
  DIORawMustLock, DIOMustLock, TWIMustLock, 0,
  SPIMustLock
};

unsigned U32(char *str) {
  unsigned u32;
  OptionUnsigned(str,&u32,0);
  return u32;
}

int State(char *str) {
  if (!strcasecmp(str,"INPUT")) {
    return INPUT;
  } else if (!strcasecmp(str,"HIGH")) {
    return HIGH;
  } else if (!strcasecmp(str,"LOW")) {
    return LOW;
  } else return U32(str);
}

int DIONum(System *sys,char *str) {
  int number = 1;
  char *s = str;
  while (*s) {
    if (!isdigit(*s) && !isspace(*s)) break;
    s++;
  }
  if (*s) { // contains non-numeric, non-whitespace character
    return sys->MapLookup(sys,str);
  } else { // contains only numbers and whitespace
    return U32(str);
  }
}

// parse the arguments passed and modify the following cmd fields:
// Inst
// Class
// Opcode
// Extra (for locks)
// Command (not really needed since we translate it to Opcode)
int CommandParse(int argc,char **argv,tsctlCommand *cmd,int canlock) {
  char *tmp;

  // split instance from class, if specified, otherwise assume 0
  if (tmp = strchr(argv[2],':')) {
    *tmp = 0;
    cmd->Inst = strtol(tmp+1,NULL,0);
    if (cmd->Inst < 0) cmd->Inst = 0;
  } else {
    cmd->Inst = 0;
  }
  for (cmd->Class=0;Class[cmd->Class];cmd->Class++) {
    if (!strcmp(Class[cmd->Class],argv[2])) {
      break;
    }
  }
  if (!Class[cmd->Class]) return CommandErrorClass; 
  if (argc < 4) return CommandErrorArgCount;

  tmp = argv[3];
  while (*tmp) tmp++;
  tmp -= 4;
  if (!strcmp(tmp,"Lock")) {
    // Lock: Bus, Pin, DIORaw, DIO, TWI, SPI
    // Don't: System, Time, CAN
    if (cmd->Class == NetSystemClass || cmd->Class == NetTimeClass
	|| cmd->Class == NetCANClass) return CommandErrorCommandBad;
    if (argc != 5) return CommandErrorArgsBad;
    /*
      NOUNLOCK SHARED NONBLOCKING
      000 WriteLock
      001 TryWriteLock
      010 ReadLock
      011 TryReadLock
      100 FOD_WriteLock
      101 FOD_TryWriteLock
      110 FOD_ReadLock
      111 FOD_TryReadLock
    */

    if (!strcmp(argv[3],"WriteLock")) {
      cmd->Extra = 0;
    } else if (!strcmp(argv[3],"TryWriteLock")) {
      cmd->Extra = 1;
    } else if (!strcmp(argv[3],"ReadLock")) {
      cmd->Extra = 2;
    } else if (!strcmp(argv[3],"TryReadLock")) {
      cmd->Extra = 3;
    } else if (!strcmp(argv[3],"FOD_WriteLock")) {
      cmd->Extra = 4;
    } else if (!strcmp(argv[3],"FOD_TryWriteLock")) {
      cmd->Extra = 5;
    } else if (!strcmp(argv[3],"FOD_ReadLock")) {
      cmd->Extra = 6;
    } else if (!strcmp(argv[3],"FOD_TryReadLock")) {
      cmd->Extra = 7;
    } else if (!strcmp(argv[3],"WriteUnlock")) {
      cmd->Extra = 8;
    } else if (!strcmp(argv[3],"ReadUnlock")) {
      cmd->Extra = 10;
    } else {
      // note: we assume that there will never be an API command ending
      // in "Lock" that is not a Lock/Unlock command!
      return CommandErrorCommandBad;
    }
    cmd->OpCode = 0;
  } else if (!strcmp(argv[3],"Preempt")) {
    if (argc != 4) return CommandErrorArgsBad;
    cmd->OpCode = 0;
    cmd->Extra = -1;
  } else {
    // check for valid Command with correct number of args
    for (cmd->Command=0;CommandNames[cmd->Class][cmd->Command];cmd->Command++) {
      if (!strcmp(CommandNames[cmd->Class][cmd->Command],argv[3])) {
	break;
      }
    }
    if (!CommandNames[cmd->Class][cmd->Command]) return CommandErrorCommandBad;

    if (argc-4 != CommandOpts[cmd->Class][cmd->Command]) { // or <
      return CommandErrorArgsBad;
    }
    cmd->OpCode = CommandOpcode[cmd->Class][cmd->Command];
    if (CommandMustLock[cmd->Class]) {
      cmd->MustLock = CommandMustLock[cmd->Class][cmd->Command];
    } else {
      cmd->MustLock = MustNoLock;
    }
  }

}

// requires appropriate object pointer to be non-zero (else returns failure)
// modify the following cmd fields:
// Status (if object InitStatus <= 0)
// rType set to return type of command
// <anonymous typed> set to return value of command
int CommandExec(tsctlCommand *cmd,char **argv) {
  int i,j;
  Bus *bus; 
  DIO *dio;
  char **a = argv;
  const char *str;

  switch(cmd->Class) {
  case NetSystemClass:
    if (!cmd->sys) return CommandErrorNoSuchObject;
    if (cmd->sys->InitStatus <= 0) {
      cmd->Status = cmd->sys->InitStatus;
      return CommandErrorObjectInit;
    }
    switch (cmd->OpCode) {
    case NetSystem_ClassCount:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->ClassCount(cmd->sys);
      return 1;
    case NetSystem_InstanceCount:
      OptionUnsigned(argv[4],&i,0);
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->InstanceCount(cmd->sys,i);
      return 1;
    case NetSystem_APICount:
      OptionUnsigned(argv[4],&i,0);
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->APICount(cmd->sys,i);
      return 1;
    case NetSystem_LockCount:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->LockCount(cmd->sys);
      return 1;
    case NetSystem_LockHolderInfo:
      i = cmd->sys->LockCount(cmd->sys);
      cmd->rType = T_VectorInteger32;
      cmd->rVectorInteger32 = malloc((2*i+1)*sizeof(int));
      cmd->rVectorInteger32[0] = 2 * i;
      return cmd->sys->LockHolderInfo(cmd->sys,(void *)&cmd->rVectorInteger32[1]);
    case NetSystem_ConnWaitInfo:
      cmd->rType = T_VectorInteger32;
      cmd->rVectorInteger32 = malloc((1024+1)*sizeof(int));
      cmd->rVectorInteger32[0] = 
	cmd->sys->ConnWaitInfo(cmd->sys,
				   1024,(void *)&cmd->rVectorInteger32[1]);
      return 1;
    case NetSystem_CANBusGet:
      OptionUnsigned(argv[4],&i,0);
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->CANBusGet(cmd->sys,i);
      return 1;
    case NetSystem_BuildInfo:
      cmd->rType = T_BuildInfo;
      cmd->rBuildInfo = malloc(sizeof(struct BuildInfo));
      cmd->sys->BuildInfo(cmd->sys,cmd->rBuildInfo);
      return 1;
    case NetSystem_ModelId:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->ModelId(cmd->sys);
      return 1;
    case NetSystem_BaseBoardId:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->BaseBoardId(cmd->sys);
      return 1;
    case NetSystem_MapLength:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->MapLength(cmd->sys);
      return 1;
    case NetSystem_MapGet:
      cmd->rType = T_Pair;
      cmd->rPair.key = malloc(257);
      i=cmd->sys->MapGet(cmd->sys,atoi(argv[4]),cmd->rPair.key,&cmd->rPair.value);
      cmd->rPair.key[i]=0;
      return 1;
    case NetSystem_MapLookup:
      cmd->rType = T_Pair;
      cmd->rPair.key = argv[4];
      cmd->rPair.value = cmd->sys->MapLookup(cmd->sys,argv[4]);
      return 1;
    case NetSystem_MapLookupPartial:
      cmd->rType = T_Pair;
      cmd->rPair.key = malloc(257);
      j = atoi(argv[5]);
      i=cmd->sys->MapLookupPartial(cmd->sys,argv[4],j,cmd->rPair.key);
      cmd->rPair.key[i]=0;
      cmd->rPair.value = j;
      return 1;
    case NetSystem_MapAdd:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->MapAdd(cmd->sys,argv[4],atoi(argv[5]));
      return 1;
    case NetSystem_MapDelete:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->MapDelete(cmd->sys,argv[4]);
      return 1;
      
    }
    break; // case NetSystemClass of switch(cmd->Class)
  case NetBusClass:
    bus = cmd->bus;
    if (!bus) return CommandErrorNoSuchObject;
    if (bus->InitStatus <= 0) {
      cmd->Status = bus->InitStatus;
      return CommandErrorObjectInit;
    }
    if (cmd->MustLock) {
      i=cmd->bus->Lock(cmd->bus,0,cmd->MustLock==MustReadLock ? SHARED:0);
      if (i<0) return CommandErrorLock-i;
    }
    switch (cmd->OpCode) {
    case 0:
      cmd->rType = T_Integer32;
      if (cmd->Extra == -1) {
	cmd->rInteger32 = cmd->bus->Preempt(cmd->bus);
      } else if (cmd->Extra < 8) {
	cmd->rInteger32 = cmd->bus->Lock(cmd->bus,U32(a[4]),cmd->Extra);
      } else {
	cmd->rInteger32 = cmd->bus->Unlock(cmd->bus,U32(a[4]),cmd->Extra);
      }
      break;
    case NetBus_Peek8:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->Peek8(bus,U32(a[4]));
      break;
    case NetBus_Peek16:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->Peek16(bus,U32(a[4]));
      break;
    case NetBus_Peek32:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->Peek32(bus,U32(a[4]));
      break;
    case NetBus_BitGet8:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->BitGet8(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitGet16:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->BitGet16(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitGet32:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->BitGet32(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_Poke8: // Poke8 address value
      cmd->rType = T_Void;
      bus->Poke8(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_Poke16: // Poke32 address value
      cmd->rType = T_Void;
      bus->Poke16(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_Poke32: // Poke32 address value
      cmd->rType = T_Void;
      bus->Poke32(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitSet8: // BitSet8 address bit
      cmd->rType = T_Void;
      bus->BitSet8(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitSet16: // BitSet16 address bit
      cmd->rType = T_Void;
      bus->BitSet16(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitSet32: // BitSet32 address bit
      cmd->rType = T_Void;
      bus->BitSet32(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitClear8: // BitClear8 address bit
      cmd->rType = T_Void;
      bus->BitClear8(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitClear16: // BitClear16 address bit
      cmd->rType = T_Void;
      bus->BitClear16(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitClear32: // BitClear32 address bit
      cmd->rType = T_Void;
      bus->BitClear32(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitAssign8: // BitAssign8 address bit value
      cmd->rType = T_Void;
      bus->BitAssign8(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      break;
    case NetBus_BitAssign16: // BitAssign16 address bit value
      cmd->rType = T_Void;
      bus->BitAssign16(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      break;
    case NetBus_BitAssign32: // BitAssign32 address bit value
      cmd->rType = T_Void;
      bus->BitAssign32(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      break;
    case NetBus_Refresh: // Refresh
      cmd->rType = T_Void;
      bus->Refresh(bus);
      break;
    case NetBus_Commit: // Commit
      cmd->rType = T_Void;
      bus->Commit(bus,U32(a[4]));
      break;
    case NetBus_BitToggle8: // BitToggle8 address bit
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->BitToggle8(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitToggle16: // BitToggle16 address bit
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->BitToggle16(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_BitToggle32: // BitToggle32 address bit
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->BitToggle32(bus,U32(a[4]),U32(a[5]));
      break;
    case NetBus_Assign8X: 
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->Assign8X(bus,U32(a[4]),U32(a[5]),
					   U32(a[6]),U32(a[7]));
      break;
    case NetBus_Assign16X: 
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->Assign16X(bus,U32(a[4]),U32(a[5]),
					    U32(a[6]),U32(a[7]));
      break;
    case NetBus_Assign32X:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->Assign32X(bus,U32(a[4]),U32(a[5]),
					    U32(a[6]),U32(a[7]));
      break;
    case NetBus_BitsGet8: 
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->BitsGet8(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      break;
    case NetBus_BitsGet16: 
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->BitsGet16(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      break;
    case NetBus_BitsGet32:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->BitsGet32(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      break;
    }
    if (cmd->MustLock) {
      i=cmd->bus->Unlock(cmd->bus,0,cmd->MustLock==MustReadLock?SHARED:0);
      if (i<0) return CommandErrorLock-i;
    }
    return 1; // case NetBusClass of switch(cmd->Class)
  case NetDIOClass:
    dio = cmd->dio;
    if (!dio) return CommandErrorNoSuchObject;
    if (dio->InitStatus <= 0) {
      cmd->Status = dio->InitStatus;
      return CommandErrorObjectInit;
    }
    if (cmd->MustLock) {
      i=cmd->dio->Lock(cmd->dio,0,cmd->MustLock==MustReadLock ? SHARED:0);
      if (i<0) return CommandErrorLock-i;
    }
    switch (cmd->OpCode) {
    case 0:
      cmd->rType = T_Integer32;
      if (cmd->Extra == -1) {
	cmd->rInteger32 = cmd->dio->Preempt(cmd->dio);
      } else if (cmd->Extra < 8) {
	cmd->rInteger32 = cmd->dio->Lock(cmd->dio,U32(a[4]),cmd->Extra);
      } else {
	cmd->rInteger32 = cmd->dio->Unlock(cmd->dio,U32(a[4]),cmd->Extra);
      }
      break;
    case NetDIO_Refresh:
      cmd->rType = T_Void;
      dio->Refresh(dio);
      break;
    case NetDIO_Commit:
      cmd->rType = T_Void;
      dio->Commit(dio,U32(a[4]));
      break;
    case NetDIO_Set:
      cmd->rType = T_Void;
      dio->Set(dio,DIONum(cmd->sys,argv[4]),State(argv[5]));
      break;
    case NetDIO_Get:
      cmd->rType = T_VectorInteger8; 
      str = DIOValueString(dio->Get(dio,DIONum(cmd->sys,argv[4])));
      i = strlen(str);
      cmd->rVectorInteger8 = malloc(i+2);
      cmd->rVectorInteger8[0] = i;
      strcpy(cmd->rVectorInteger8+1,str);
      break;
    case NetDIO_SetAsync:
      cmd->rType = T_Void;
      dio->SetAsync(dio,DIONum(cmd->sys,argv[4]),State(argv[5]));
      break;
    case NetDIO_GetAsync:
      cmd->rType = T_VectorInteger8; 
      str = DIOValueString(dio->GetAsync(dio,DIONum(cmd->sys,argv[4])));
      i = strlen(str);
      cmd->rVectorInteger8 = malloc(i+2);
      cmd->rVectorInteger8[0] = i;
      strcpy(cmd->rVectorInteger8+1,str);
      break;
    case NetDIO_Wait:

      break;
    case NetDIO_Count:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = dio->Count(dio); 
      break;
    case NetDIO_Capabilities:
      break;
    }
    if (cmd->MustLock) {
      i=cmd->dio->Unlock(cmd->dio,0,cmd->MustLock==MustReadLock?SHARED:0);
      if (i<0) return CommandErrorLock-i;
    }
    return 1; // case NetDIOClass of switch(cmd->Class)
  }
}


int tsctlInterpret(int argc,char *argv[],int autolock) {
  tsctlCommand cmd;
  int i,ret;

  memset(&cmd,0,sizeof(cmd));
  ret = CommandParse(argc,argv,&cmd,0);
  if (!autolock) cmd.MustLock = 0;

  switch (ret) {
  case CommandErrorClass:
    fprintf(stderr,"Error: unrecognized Class\n");
    fprintf(stderr,"Select from ");
    for (cmd.Class=0;Class[cmd.Class];cmd.Class++) {
      fprintf(stderr,"%s",Class[cmd.Class]);
      if (Class[cmd.Class+1]) fprintf(stderr,", ");
    }
    fprintf(stderr,"\n");
    return 1;
  case CommandErrorArgCount:
    fprintf(stderr,"Valid commands for %s are:\n",argv[2]);
    for (cmd.Command=0;CommandNames[cmd.Class][cmd.Command];cmd.Command++) {
      fprintf(stderr,"%s ",CommandNames[cmd.Class][cmd.Command]);
    }
    fprintf(stderr,"\n");
    return 1;
  case CommandErrorCommandBad:
    fprintf(stderr,"Error: unrecognized Command \"%s\"\n",argv[3]);
    fprintf(stderr,"Select from ");
    for (cmd.Command=0;CommandNames[cmd.Class][cmd.Command];cmd.Command++) {
      fprintf(stderr,"%s",CommandNames[cmd.Class][cmd.Command]);
      if (CommandNames[cmd.Class][cmd.Command+1]) fprintf(stderr,", ");
    }
    fprintf(stderr,"\n");
    return 1;    
  case CommandErrorArgsBad:
    fprintf(stderr,"Usage: tsctl %s %s %s %s\n",argv[1],argv[2],argv[3],
	    CommandDesc[cmd.Class][cmd.Command]);
    return 1;
  }

  if (cmd.Class != NetSystemClass) {
    cmd.sys = SystemInit(0);
  }
  switch(cmd.Class) {
  case NetSystemClass: cmd.sys = SystemInit(cmd.Inst); break;
  case NetBusClass:    cmd.bus = BusInit(cmd.Inst); break;
  case NetTimeClass:   cmd.time = TimeInit(cmd.Inst); break;
  case NetPinClass:    cmd.pin = PinInit(cmd.Inst); break;
  case NetDIORawClass: cmd.dioraw = DIORawInit(cmd.Inst); break;
  case NetDIOClass:    cmd.dio = DIOInit(cmd.Inst); break;
  case NetTWIClass:    cmd.twi = TWIInit(cmd.Inst); break;
  case NetCANClass:    cmd.can = CANInit(cmd.Inst); break;
  case NetSPIClass:    cmd.spi = SPIInit(cmd.Inst); break;
  //case NetEDIOClass: cmd.edio = EDIOInit(cmd.Inst); break;
  case NetAIOClass:    cmd.aio = AIOInit(cmd.Inst); break;
  }

  ret = CommandExec(&cmd,argv);

  switch (ret) {
  case CommandErrorNoSuchObject:
    printf("Error: %s %d not defined for this architecture\n",
	   Class[cmd.Class],cmd.Inst);
    return 1;
  case CommandErrorObjectInit:
    printf("Error %d\n",-cmd.Status);
    return 1;
  }

  switch (cmd.rType) {
  case T_Void:
    hprintf(1,"null");
    break;
  case T_Integer32:
    hprintf(1,"%d",cmd.rInteger32);
    break;
  case T_VectorInteger32:
    hprintf(1,"[");
    for (i=1;i<=cmd.rVectorInteger32[0];i++) {
      if (i == cmd.rVectorInteger32[0]) {
	hprintf(1,"%d",cmd.rVectorInteger32[i]);
      } else {
	hprintf(1,"%d,",cmd.rVectorInteger32[i]);
      }
    }
    hprintf(1,"]");
    free(cmd.rVectorInteger32);
    break;
  case T_BuildInfo: 
    break;
  case T_VectorInteger8:
    hprintf(1,"\"");
    write(1,cmd.rVectorInteger8+1,cmd.rVectorInteger8[0]);
    hprintf(1,"\"");
    free(cmd.rVectorInteger8);
    break;
  case T_Pair:
    hprintf(1,"{ \"%s\": %d }",cmd.rPair.key,cmd.rPair.value);
    free(cmd.rPair.key);
    break;
  }
  hprintf(1,"\n");
}

int tsctlProcessFile(FILE *f) {
  char cmd[80],*str,*argv[16];
  int argc,len,quoting;

  while (!feof(f)) {
    if (!fgets(cmd,78,f)) return 0;
    if (!strncasecmp(cmd,"end",3)) return 0;
    len = strlen(cmd);
    str = cmd; argc = 2; quoting = 0;
    if (cmd[len-1] == '\n') cmd[len-1] = 0;
    argv[argc++] = str++;
    while (*str && argc < 16) {
      if (*str == ' ' && !quoting) {
	argv[argc++] = str+1;
	*str = 0;
      } else if (*str == '"' || *str == '\'') {
	quoting = !quoting;
      }
      str++;
    }
    tsctlInterpret(argc,argv,0);
  }
  return 0;
}

int main(int argc,char *argv[]) {
  // argument "--" to read commands from stdin
  // argument "-f file" to read commands from file
  // check for minimum parms for client invocation

  if (argc == 2 && !strcmp(argv[1],"--")) {
    tsctlProcessFile(stdin);
  } else if (argc == 3 && !strcmp(argv[1],"-f")) {
    FILE *f = fopen(argv[2],"r");
    if (!f) {
      fprintf(stderr,"Error opening %s: %m\n",argv[2]);
      return 1;
    }
    tsctlProcessFile(f);
    fclose(f);
  } else if (argc < 3) {
    fprintf(stderr,"Usage:\n");
    fprintf(stderr,"\ttsctl --server\n");
    fprintf(stderr,"\ttsctl <Host> <Class[:Instance]> <Command> [Args...]\n");
    return 1;
  } else {
    tsctlInterpret(argc,argv,1);
  }
  return 0;
}
